@@ -213,6 +213,49 @@ where
         self.exp_backlog.push(removed);
     }
 
+    unsafe fn raw_insert(
+        &mut self,
+        k: K,
+        v: V,
+        init: P::Info,
+    ) -> (Bucket<(K, V, Storage<P::Storage>)>, Option<V>) {
+        // try to process single backlog when on every mutable state.
+        self.process_single_backlog();
+
+        let hash = make_insert_hash::<K, H>(&self.hash_builder, &k);
+
+        // initialize storage with info value.
+        let s = self.exp_policy.init_storage(init);
+        let storage = Storage::new(s, self.exp_bucket_table.acquire_slot());
+
+        if let Some((_, old_v, old_s)) = self.table.get_mut(hash, equivalent_key(&k)) {
+            let bucket = self.exp_bucket_table.get(old_s.entry_id).unwrap();
+
+            // we need to mark it as removed. but not releasing id.
+            // and assign a new id to track it.
+            self.exp_bucket_table.set_bucket(old_s.entry_id, None);
+            self.exp_bucket_table
+                .set_bucket(storage.entry_id, Some(bucket.clone()));
+
+            let old_s = mem::replace(old_s, storage);
+            let old_v = mem::replace(old_v, v);
+            if self.exp_policy.is_expired(old_s.entry_id, &old_s.storage)
+                || unlikely(old_s.is_removed())
+            {
+                (bucket, None)
+            } else {
+                (bucket, Some(old_v))
+            }
+        } else {
+            let bucket = match self.table.try_insert_no_grow(hash, (k, v, storage)) {
+                Ok(bucket) => bucket,
+                Err(v) => self.grow_and_insert(hash, v),
+            };
+
+            (bucket, None)
+        }
+    }
+
     /// Returns a reference to the value corresponding to the key.
     ///
     /// If the entry is expired, returns `None`
@@ -404,41 +447,7 @@ where
     /// ```
     #[inline]
     pub fn insert(&mut self, k: K, v: V, init: P::Info) -> Option<V> {
-        // try to process single backlog when on every mutable state.
-        self.process_single_backlog();
-
-        let hash = make_insert_hash::<K, H>(&self.hash_builder, &k);
-
-        // initialize storage with info value.
-        let s = self.exp_policy.init_storage(init);
-        let storage = Storage::new(s, self.exp_bucket_table.acquire_slot());
-
-        let (bucket, old_v) =
-            if let Some((_, old_v, old_s)) = self.table.get_mut(hash, equivalent_key(&k)) {
-                let bucket = self.exp_bucket_table.get(old_s.entry_id).unwrap();
-
-                // we need to mark it as removed. but not releasing id.
-                // and assign a new id to track it.
-                self.exp_bucket_table.set_bucket(old_s.entry_id, None);
-                self.exp_bucket_table
-                    .set_bucket(storage.entry_id, Some(bucket.clone()));
-
-                let old_s = mem::replace(old_s, storage);
-                if self.exp_policy.is_expired(old_s.entry_id, &old_s.storage)
-                    || unlikely(old_s.is_removed())
-                {
-                    (bucket, None)
-                } else {
-                    (bucket, Some(mem::replace(old_v, v)))
-                }
-            } else {
-                let bucket = match self.table.try_insert_no_grow(hash, (k, v, storage)) {
-                    Ok(bucket) => bucket,
-                    Err(v) => self.grow_and_insert(hash, v),
-                };
-
-                (bucket, None)
-            };
+        let (bucket, old_v) = unsafe { self.raw_insert(k, v, init) };
 
         let (_, _, s) = unsafe { bucket.as_mut() };
         self.exp_bucket_table.set_bucket(s.entry_id, Some(bucket));
@@ -586,7 +595,7 @@ where
     /// Shrinks the capacity of the `HashMap` as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.
     ///
     /// This function updates internal bucket id which tracks [`EntryId`] because of bucket relocation.
-    /// 
+    ///
     /// # Examples
     ///
     /// ```
@@ -614,7 +623,7 @@ where
     /// Shrinks the capacity of the `HashMap` with a lower limit. It will drop down no lower than the supplied limit while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.
     ///
     /// This function updates internal bucket id which tracks [`EntryId`] because of bucket relocation.
-    /// 
+    ///
     /// # Examples
     ///
     /// ```
@@ -642,9 +651,30 @@ where
         self.update_bucket_id();
     }
 
+    pub fn entry(&mut self, key: K) -> Entry<'_, K, V, P, H> {
+        let hash = make_insert_hash(&self.hash_builder, &key);
+
+        if let Some(elem) = self.table.find(hash, equivalent_key(&key)) {
+            let s = unsafe { &elem.as_ref().2 };
+            if !self.exp_policy.is_expired(s.entry_id, &s.storage) || !s.is_removed() {
+                return Entry::Occupied(OccupiedEntry {
+                    hash,
+                    key: Some(key),
+                    elem,
+                    table: self,
+                });
+            }
+        }
+        Entry::Vacant(VacantEntry {
+            hash,
+            key,
+            table: self,
+        })
+    }
+
     /// Clears the `Hashmap`, returning all key-value pairs as an iterator. Keeps the allocated memory for reuse.
     ///
-    /// When drop, this function also triggers internal [`ExpirePolicy::clear`].
+    /// When drop, this function also triggers internal [`ExpirePolicy::clear()`].
     ///
     /// # Examples
     ///
@@ -708,7 +738,7 @@ where
 
     /// Returns the exact number of elements in the `HashMap`.
     ///
-    /// This functions is accurate than [`HashMap::len_approx`] but uses slower algorithm `O(n)`.
+    /// This functions is accurate than [`HashMap::len_approx()`] but uses slower algorithm `O(n)`.
     /// # Examples
     ///
     /// ```
@@ -729,11 +759,11 @@ where
     }
 
     /// Returns the number of elements in the `HashMap`.
-    /// 
+    ///
     /// This function may contains count of elements in the `HashMap` that is not actually removed from internal table.
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```
     /// use endorphin::policy::LazyFixedTTLPolicy;
     /// use endorphin::HashMap;
@@ -745,7 +775,7 @@ where
     /// assert_eq!(cache.len_approx(), 0);
     /// cache.insert(0, "a", ());
     /// assert_eq!(cache.len_approx(), 1);
-    /// 
+    ///
     /// sleep(Duration::from_millis(10));
     /// assert_eq!(cache.len(), 0);
     /// assert_eq!(cache.len_approx(), 1);
@@ -817,7 +847,7 @@ where
     /// cache.insert(0, "a", ());
     /// cache.insert(1, "b", ());
     /// cache.insert(2, "c", ());
-    /// 
+    ///
     /// for (k, v) in cache.iter() {
     ///     println!("key: {}, val: {}", k, v);
     /// }
@@ -834,9 +864,9 @@ where
     }
 
     /// An iterator visiting all key-value pairs in arbitrary order, with mutable references to the values. The iterator element type is `(&'a K, &'a mut V)`.
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```
     /// use endorphin::policy::LazyFixedTTLPolicy;
     /// use endorphin::HashMap;
@@ -848,11 +878,11 @@ where
     /// cache.insert(0, 0, ());
     /// cache.insert(1, 1, ());
     /// cache.insert(2, 2, ());
-    /// 
+    ///
     /// for (_, v) in cache.iter_mut() {
     ///     *v *= 2;
     /// }
-    /// 
+    ///
     /// for (k, v) in cache.iter() {
     ///     println!("key: {}, val: {}", k, v);
     /// }
@@ -869,9 +899,9 @@ where
     }
 
     /// An iterator visiting all values in arbitrary order. The iterator element type is `&'a V`.
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```
     /// use endorphin::policy::LazyFixedTTLPolicy;
     /// use endorphin::HashMap;
@@ -883,7 +913,7 @@ where
     /// cache.insert(0, "a", ());
     /// cache.insert(1, "b", ());
     /// cache.insert(2, "c", ());
-    /// 
+    ///
     /// for v in cache.values() {
     ///     println!("{}", v);
     /// }
@@ -894,9 +924,9 @@ where
     }
 
     /// An iterator visiting all values mutably in arbitrary order. The iterator element type is `&'a mut V`.
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```
     /// use endorphin::policy::LazyFixedTTLPolicy;
     /// use endorphin::HashMap;
@@ -908,12 +938,12 @@ where
     /// cache.insert(0, 0, ());
     /// cache.insert(1, 1, ());
     /// cache.insert(2, 2, ());
-    /// 
+    ///
     /// for v in cache.values_mut() {
     ///     *v = *v + 6;
     /// }
-    /// 
-    /// for v in map.values() {
+    ///
+    /// for v in cache.values() {
     ///     println!("{}", v);
     /// }
     /// ```
@@ -925,9 +955,9 @@ where
     }
 
     /// An iterator visiting all keys in arbitrary order. The iterator element type is `&'a K`.
-    /// 
+    ///
     /// # Examples
-    /// 
+    ///
     /// ```
     /// use endorphin::policy::LazyFixedTTLPolicy;
     /// use endorphin::HashMap;
@@ -939,7 +969,7 @@ where
     /// cache.insert(0, "a", ());
     /// cache.insert(1, "b", ());
     /// cache.insert(2, "c", ());
-    /// 
+    ///
     /// for v in cache.keys() {
     ///     println!("{}", v);
     /// }
@@ -1107,6 +1137,266 @@ where
     }
 }
 
+pub enum Entry<'a, K, V, P, H>
+where
+    P: ExpirePolicy,
+{
+    Occupied(OccupiedEntry<'a, K, V, P, H>),
+    Vacant(VacantEntry<'a, K, V, P, H>),
+}
+
+impl<'a, K, V, P, H> Entry<'a, K, V, P, H>
+where
+    P: ExpirePolicy,
+    K: Eq + Hash,
+    H: BuildHasher,
+{
+    pub fn insert(self, value: V, init: P::Info) -> OccupiedEntry<'a, K, V, P, H> {
+        match self {
+            Entry::Occupied(mut entry) => {
+                entry.insert(value, init);
+                entry
+            }
+            Entry::Vacant(entry) => entry.insert_entry(value, init),
+        }
+    }
+
+    pub fn or_insert(self, default: V, init: P::Info) -> &'a mut V {
+        match self {
+            Entry::Occupied(entry) => entry.into_mut(),
+            Entry::Vacant(entry) => entry.insert(default, init),
+        }
+    }
+
+    pub fn or_insert_with<F: FnOnce() -> V>(self, default: F, init: P::Info) -> &'a mut V {
+        match self {
+            Entry::Occupied(entry) => entry.into_mut(),
+            Entry::Vacant(entry) => entry.insert(default(), init),
+        }
+    }
+
+    pub fn or_insert_with_key<F: FnOnce(&K) -> V>(self, default: F, init: P::Info) -> &'a mut V {
+        match self {
+            Entry::Occupied(entry) => entry.into_mut(),
+            Entry::Vacant(entry) => {
+                let value = default(entry.key());
+                entry.insert(value, init)
+            }
+        }
+    }
+
+    pub fn key(&self) -> &K {
+        match *self {
+            Entry::Occupied(ref entry) => entry.key(),
+            Entry::Vacant(ref entry) => entry.key(),
+        }
+    }
+
+    pub fn and_modify<F>(self, f: F) -> Self
+    where
+        F: FnOnce(&mut V),
+    {
+        match self {
+            Entry::Occupied(mut entry) => {
+                f(entry.get_mut());
+                Entry::Occupied(entry)
+            }
+            Entry::Vacant(entry) => Entry::Vacant(entry),
+        }
+    }
+
+    pub fn and_replace_entry_with<F>(self, f: F) -> Self
+    where
+        F: FnOnce(&K, V) -> Option<V>,
+    {
+        match self {
+            Entry::Occupied(entry) => entry.replace_entry_with(f),
+            Entry::Vacant(_) => self,
+        }
+    }
+}
+
+pub struct OccupiedEntry<'a, K, V, P, H>
+where
+    P: ExpirePolicy,
+{
+    hash: u64,
+    key: Option<K>,
+    elem: Bucket<(K, V, Storage<P::Storage>)>,
+    table: &'a mut HashMap<K, V, P, H>,
+}
+
+impl<'a, K, V, P, H> OccupiedEntry<'a, K, V, P, H>
+where
+    P: ExpirePolicy,
+    K: Eq + Hash,
+    H: BuildHasher,
+{
+    pub fn key(&self) -> &K {
+        unsafe { &self.elem.as_ref().0 }
+    }
+
+    pub fn remove_entry(self) -> (K, V) {
+        let key = unsafe { &self.elem.as_ref().0 };
+        self.table.remove_entry(key).unwrap()
+    }
+
+    pub fn get(&self) -> &V {
+        unsafe { &self.elem.as_ref().1 }
+    }
+
+    pub fn get_mut(&mut self) -> &mut V {
+        unsafe { &mut self.elem.as_mut().1 }
+    }
+
+    pub fn into_mut(self) -> &'a mut V {
+        unsafe { &mut self.elem.as_mut().1 }
+    }
+
+    pub fn insert(&mut self, value: V, init: P::Info) -> V {
+        let k = unsafe { &self.elem.as_ref().0 };
+
+        let s = self.table.exp_policy.init_storage(init);
+        let storage = Storage::new(s, self.table.exp_bucket_table.acquire_slot());
+
+        let (_, old_v, old_s) = self
+            .table
+            .table
+            .get_mut(self.hash, equivalent_key(k))
+            .unwrap();
+        let bucket = self.table.exp_bucket_table.get(old_s.entry_id).unwrap();
+
+        self.table.exp_bucket_table.set_bucket(old_s.entry_id, None);
+        self.table
+            .exp_bucket_table
+            .set_bucket(storage.entry_id, Some(bucket.clone()));
+
+        let _ = mem::replace(old_s, storage);
+        let old_v = mem::replace(old_v, value);
+
+        let (_, _, s) = unsafe { bucket.as_mut() };
+        self.table
+            .exp_bucket_table
+            .set_bucket(s.entry_id, Some(bucket));
+
+        self.table
+            .handle_status(self.table.exp_policy.on_insert(s.entry_id, &mut s.storage));
+
+        old_v
+    }
+
+    pub fn remove(self) -> V {
+        self.table.remove(unsafe { &self.elem.as_ref().0 }).unwrap()
+    }
+
+    pub fn replace_entry(self, value: V) -> (K, V) {
+        let entry = unsafe { self.elem.as_mut() };
+
+        let old_key = mem::replace(&mut entry.0, self.key.unwrap());
+        let old_value = mem::replace(&mut entry.1, value);
+
+        (old_key, old_value)
+    }
+
+    pub fn replace_key(self) -> K {
+        let entry = unsafe { self.elem.as_mut() };
+        mem::replace(&mut entry.0, self.key.unwrap())
+    }
+
+    pub fn replace_entry_with<F>(self, f: F) -> Entry<'a, K, V, P, H>
+    where
+        F: FnOnce(&K, V) -> Option<V>,
+    {
+        unsafe {
+            let mut spare_key = None;
+
+            self.table
+                .table
+                .replace_bucket_with(self.elem.clone(), |(key, value, policy)| {
+                    if let Some(new_value) = f(&key, value) {
+                        Some((key, new_value, policy))
+                    } else {
+                        spare_key = Some(key);
+                        None
+                    }
+                });
+
+            if let Some(key) = spare_key {
+                Entry::Vacant(VacantEntry {
+                    hash: self.hash,
+                    key,
+                    table: self.table,
+                })
+            } else {
+                Entry::Occupied(self)
+            }
+        }
+    }
+}
+
+pub struct VacantEntry<'a, K, V, P, H>
+where
+    P: ExpirePolicy,
+{
+    hash: u64,
+    key: K,
+    table: &'a mut HashMap<K, V, P, H>,
+}
+
+impl<'a, K, V, P, H> VacantEntry<'a, K, V, P, H>
+where
+    P: ExpirePolicy,
+    K: Eq,
+{
+    pub fn key(&self) -> &K {
+        &self.key
+    }
+
+    pub fn into_key(self) -> K {
+        self.key
+    }
+
+    pub fn insert(self, value: V, init: P::Info) -> &'a mut V
+    where
+        K: Hash,
+        H: BuildHasher,
+    {
+        let bucket = unsafe { self.table.raw_insert(self.key, value, init).0 };
+        let (_, _, s) = unsafe { bucket.as_mut() };
+        self.table
+            .exp_bucket_table
+            .set_bucket(s.entry_id, Some(bucket.clone()));
+
+        self.table
+            .handle_status(self.table.exp_policy.on_insert(s.entry_id, &mut s.storage));
+
+        unsafe { &mut bucket.as_mut().1 }
+    }
+
+    pub fn insert_entry(self, value: V, init: P::Info) -> OccupiedEntry<'a, K, V, P, H>
+    where
+        K: Hash,
+        H: BuildHasher,
+    {
+        let elem = unsafe { self.table.raw_insert(self.key, value, init).0 };
+
+        let (_, _, s) = unsafe { elem.as_mut() };
+        self.table
+            .exp_bucket_table
+            .set_bucket(s.entry_id, Some(elem.clone()));
+
+        self.table
+            .handle_status(self.table.exp_policy.on_insert(s.entry_id, &mut s.storage));
+
+        OccupiedEntry {
+            hash: self.hash,
+            key: None,
+            elem,
+            table: self.table,
+        }
+    }
+}
+
 #[cfg(test)]
 mod test_map {
     use super::HashMap;
@@ -1125,21 +1415,21 @@ mod test_map {
 
         type Storage = ();
 
-        fn init_storage(&self, info: Self::Info) -> Self::Storage {
+        fn init_storage(&self, _: Self::Info) -> Self::Storage {
             ()
         }
 
         fn clear(&mut self) {}
 
-        fn is_expired(&self, entry: EntryId, storage: &Self::Storage) -> bool {
+        fn is_expired(&self, _: EntryId, _: &Self::Storage) -> bool {
             false
         }
 
-        fn on_access(&self, entry: EntryId, storage: &Self::Storage) -> Command {
+        fn on_access(&self, _: EntryId, _: &Self::Storage) -> Command {
             Command::Noop
         }
 
-        fn on_insert(&self, entry: EntryId, storage: &Self::Storage) -> Command {
+        fn on_insert(&self, _: EntryId, _: &Self::Storage) -> Command {
             Command::Noop
         }
 
@@ -1163,6 +1453,13 @@ mod test_map {
         assert_eq!(map.capacity(), 0);
     }
 
+    #[test]
+    fn test_with_capacity() {
+        let map = HashMap::<i32, i32, _>::with_capacity(20, MockPolicy::new());
+
+        assert!(map.capacity() >= 20);
+    }
+
     #[test]
     fn test_insert() {
         let mut map = HashMap::new(MockPolicy::new());
@@ -1261,6 +1558,10 @@ mod test_map {
         assert_eq!(map.remove(&0).unwrap(), 1);
 
         assert_eq!(map.contains_key(&0), false);
+
+        assert_eq!(map.contains_key(&10), false);
+
+        assert_eq!(map.remove(&10), None);
     }
 
     #[test]
@@ -1391,4 +1692,355 @@ mod test_map {
         map.remove(&2);
         assert_eq!(map.len(), 0);
     }
-}
\ No newline at end of file
+
+    #[test]
+    fn test_entry() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(0, 0, ());
+
+        match map.entry(0) {
+            Entry::Occupied(_) => assert!(true),
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        match map.entry(1) {
+            Entry::Occupied(_) => unreachable!(),
+            Entry::Vacant(_) => assert!(true),
+        }
+    }
+
+    #[test]
+    fn test_entry_insert() {
+        let mut map = HashMap::new(MockPolicy::new());
+
+        let mut entry = map.entry(0).insert("0", ());
+        assert_eq!(entry.get(), &"0");
+
+        entry.insert("1", ());
+        assert_eq!(map.get(&0).unwrap(), &"1");
+    }
+
+    #[test]
+    fn test_entry_or_insert() {
+        let mut map = HashMap::new(MockPolicy::new());
+
+        assert_eq!(map.entry(0).or_insert(0, ()), &0);
+        assert_eq!(map.get(&0).unwrap(), &0);
+
+        *map.entry(0).or_insert(0, ()) += 5;
+        assert_eq!(map.entry(0).or_insert(0, ()), &5);
+        assert_eq!(map.get(&0).unwrap(), &5);
+    }
+
+    #[test]
+    fn test_entry_or_insert_with() {
+        let five = || 5;
+
+        let mut map = HashMap::new(MockPolicy::new());
+
+        assert_eq!(map.entry(0).or_insert_with(five, ()), &five());
+
+        *map.entry(0).or_insert_with(five, ()) *= 2;
+
+        assert_eq!(map.entry(0).or_insert_with(five, ()), &(five() * 2));
+        assert_eq!(map.get(&0).unwrap(), &(five() * 2));
+    }
+
+    #[test]
+    fn test_entry_or_insert_with_key() {
+        let mut map = HashMap::new(MockPolicy::new());
+
+        assert_eq!(map.entry(0).or_insert_with_key(|x| x + 1, ()), &1);
+        *map.entry(0).or_insert_with_key(|x| x + 1, ()) += 5;
+
+        assert_eq!(map.entry(0).or_insert_with_key(|x| x + 1, ()), &6);
+        assert_eq!(map.get(&0).unwrap(), &6);
+    }
+
+    #[test]
+    fn test_entry_key() {
+        let mut map = HashMap::<_, u32, _>::new(MockPolicy::new());
+
+        for i in 0..100 {
+            assert_eq!(map.entry(i).key(), &i);
+        }
+    }
+
+    #[test]
+    fn test_entry_and_modify() {
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(0, 0, ());
+
+        let now = map
+            .entry(0)
+            .and_modify(|v| *v = *v + 5)
+            .and_modify(|v| *v = *v * 10);
+        assert_eq!(now.or_insert(0, ()), &50);
+        assert_eq!(map.get(&0).unwrap(), &50);
+    }
+
+    #[test]
+    fn test_entry_and_replace_entry_with() {
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        map.entry(1).and_replace_entry_with(|k, v| Some(k + v));
+
+        assert_eq!(map.get(&1).unwrap(), &11);
+    }
+
+    #[test]
+    fn test_occupied_entry_key() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        match map.entry(1) {
+            Entry::Occupied(entry) => assert_eq!(entry.key(), &1),
+            Entry::Vacant(_) => unreachable!(),
+        }
+    }
+
+    #[test]
+    fn test_occupied_entry_remove_entry() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        match map.entry(1) {
+            Entry::Occupied(entry) => assert_eq!(entry.remove_entry(), (1, 10)),
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        assert_eq!(map.get(&1), None);
+    }
+
+    #[test]
+    fn test_occupied_entry_get() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        match map.entry(1) {
+            Entry::Occupied(entry) => assert_eq!(entry.get(), &10),
+            Entry::Vacant(_) => unreachable!(),
+        }
+    }
+
+    #[test]
+    fn test_occupied_entry_get_mut() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        match map.entry(1) {
+            Entry::Occupied(mut entry) => {
+                let v = entry.get_mut();
+                assert_eq!(v, &10);
+                *v += 10;
+
+                let v = entry.get_mut(); // not moved
+            }
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        assert_eq!(map.get(&1).unwrap(), &20);
+    }
+
+    #[test]
+    fn test_occupied_entry_into_mut() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        match map.entry(1) {
+            Entry::Occupied(entry) => {
+                let v = entry.into_mut();
+                assert_eq!(v, &10);
+                *v += 10;
+            }
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        assert_eq!(map.get(&1).unwrap(), &20);
+    }
+
+    #[test]
+    fn test_occupied_entry_insert() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        assert_eq!(map.get(&1).unwrap(), &10);
+
+        match map.entry(1) {
+            Entry::Occupied(mut entry) => assert_eq!(entry.insert(20, ()), 10),
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        assert_eq!(map.get(&1).unwrap(), &20);
+    }
+
+    #[test]
+    fn test_occupied_entry_remove() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+        map.insert(1, 10, ());
+
+        match map.entry(1) {
+            Entry::Occupied(entry) => assert_eq!(entry.remove(), 10),
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        assert_eq!(map.get(&1), None);
+    }
+
+    #[test]
+    fn test_occupied_entry_replace_entry() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+
+        map.insert(0, Vec::<u32>::new(), ());
+
+        match map.entry(0) {
+            Entry::Occupied(entry) => {
+                let old = entry.replace_entry(Vec::with_capacity(10));
+                assert_eq!(old.0, 0);
+                assert_eq!(old.1.capacity(), 0);
+            }
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        assert_eq!(map.get(&0).unwrap().capacity(), 10);
+    }
+
+    #[test]
+    fn test_occupied_entry_replace_key() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+
+        let vec = Vec::<u32>::new();
+
+        map.insert(vec.clone(), (), ());
+
+        match map.entry(Vec::with_capacity(10)) {
+            Entry::Occupied(entry) => {
+                let old = entry.replace_key();
+                assert_eq!(old.capacity(), 0);
+            }
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        match map.entry(Vec::with_capacity(3)) {
+            Entry::Occupied(entry) => assert_eq!(entry.key().capacity(), 10),
+            Entry::Vacant(_) => unreachable!(),
+        }
+    }
+
+    #[test]
+    fn test_occupied_entry_replace_entry_with() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+
+        map.insert(Vec::<u32>::new(), 10, ());
+
+        let entry = match map.entry(Vec::new()) {
+            Entry::Occupied(entry) => entry.replace_entry_with(|k, v| {
+                assert_eq!(k.capacity(), 0);
+                assert_eq!(v, 10);
+                Some(v * 10)
+            }),
+            Entry::Vacant(_) => unreachable!(),
+        };
+
+        match entry {
+            Entry::Occupied(entry) => {
+                assert_eq!(entry.key().capacity(), 0);
+                assert_eq!(entry.get(), &100)
+            }
+            Entry::Vacant(_) => unreachable!(),
+        }
+
+        let entry = match map.entry(Vec::new()) {
+            Entry::Occupied(entry) => entry.replace_entry_with(|k, v| {
+                assert_eq!(k.capacity(), 0);
+                assert_eq!(v, 100);
+                None
+            }),
+            Entry::Vacant(_) => unreachable!(),
+        };
+
+        match entry {
+            Entry::Occupied(_) => unreachable!(),
+            Entry::Vacant(_) => {
+                assert_eq!(map.get(&Vec::new()), None)
+            }
+        };
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_occupied_entry_insert_result_replace_key() {
+        let mut map = HashMap::new(MockPolicy::new());
+
+        map.entry(0).insert(0, ()).replace_key();
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_occupied_entry_insert_result_replace_entry() {
+        let mut map = HashMap::new(MockPolicy::new());
+
+        map.entry(0).insert(0, ()).replace_entry(1);
+    }
+
+    #[test]
+    fn test_vacant_entry_key() {
+        use super::Entry;
+        let mut map = HashMap::<u32, u32, _>::new(MockPolicy::new());
+
+        match map.entry(0) {
+            Entry::Occupied(_) => unreachable!(),
+            Entry::Vacant(entry) => {
+                assert_eq!(entry.key(), &0)
+            }
+        }
+    }
+
+    #[test]
+    fn test_vacant_entry_insert() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+
+        assert_eq!(map.get(&0), None);
+
+        match map.entry(0) {
+            Entry::Occupied(_) => unreachable!(),
+            Entry::Vacant(entry) => {
+                let v = entry.insert(10, ());
+                *v += 20;
+            }
+        }
+
+        assert_eq!(map.get(&0).unwrap(), &30);
+    }
+
+    #[test]
+    fn test_vacant_entry_insert_entry() {
+        use super::Entry;
+        let mut map = HashMap::new(MockPolicy::new());
+
+        assert_eq!(map.get(&0), None);
+
+        let entry = match map.entry(0) {
+            Entry::Occupied(_) => unreachable!(),
+            Entry::Vacant(entry) => {
+                entry.insert_entry(33, ())
+            }
+        };
+
+        assert_eq!(entry.get(), &33);
+        assert_eq!(map.get(&0).unwrap(), &33);
+    }
+}
